import flet as ft
from datetime import datetime
from random import choice
from pages import file_handler

def get_time():
    t = datetime.now()
    return [t.year, t.month, t.day, t.hour, t.minute, t.second]

def random_emoji():
    return choice("🍎🍐🍊🍋🍌🍉🍇🍓🍈🍒🍑🍍🥝🥭🥑🍅🍆🥒🥕🥬🌽🥔🍠🌰🥜🍯🥐🍞🥖🥨🥯🧀🥚🍳🥓🧄🧅🥞🧇🍤🍗🍖🍕🌭🍔🍟🥙🌮🌯🥗🥘🍝🍜🦪🍲🍥🍣🍱🍛🍚🧆🍙🍘🍢🍡🍧🍨🍦🍰🎂🍮🍭🍬🍫🍿🍩🍪🥮🧁🥛🧈🍼☕🍵🍶🍺🍻🥂🍷🥃🍸🍹🍾🧉🧃🧊🧂🥄🍴")


class Nahrung(ft.UserControl):
    def __init__(self) -> None:
        super().__init__()
        self.settings, self.history = file_handler.load(self)

        self.kjtext = ft.Text("0", size=25)
        self.kcaltext = ft.Text("0", size=30)
        self.kcaltextfield = ft.TextField(keyboard_type="NUMBER", expand=True, on_change=self.reseterrorinkcaltextfield, on_submit=self.editkcal)

        # Initialize history list
        self.historylist = []
        self.update_history()

        # Initialize controls
        self.main_container = ft.Container(
            content=ft.Column(
                [
                    ft.Row(
                        [
                            ft.Text("kjoule:", size=20),
                            self.kjtext,
                            ft.Text(f"/{round(self.settings['kcal_max'] * 4.184, 0)}", size=10)
                        ]
                    ),
                    ft.Row(
                        [
                            ft.Text("kcal:", size=25),
                            self.kcaltext,
                            ft.Text(f"/{self.settings['kcal_max']}", size=10)
                        ]
                    ),
                    ft.Row(
                        [
                            self.kcaltextfield,
                            ft.FloatingActionButton(icon="add", on_click=self.editkcal)
                        ]
                    ),
                ]
            ),
            bgcolor=ft.colors.INDIGO_400,
            border_radius=10,
            padding=15,
        )

        self.history_container = ft.Container(
            content=ft.Column([ft.Text("Verlauf:"),
                               ft.Column(controls=self.historylist)
                               ]),
        )

        self.controls = [self.main_container, self.history_container]

        self.load_todays_kcal()

    def reseterrorinkcaltextfield(self, e=None):
        self.kcaltextfield.error_text = ""

    def build(self):
        self.kjtext.value = str(round(int(self.kcaltext.value) * 4.184, 0))
        return ft.Column(self.controls)

    def editkcal(self, e):
        try:
            emoji = random_emoji()
            new_kcal = int(self.kcaltext.value) + int(self.kcaltextfield.value)
            self.kcaltext.value = str(new_kcal)
            self.kjtext.value = str(round(new_kcal * 4.184, 0))
            self.history.append([emoji, self.kcaltextfield.value, get_time()])
            file_handler.save(self.settings, self.history)
            self.update_history()
            self.reseterrorinkcaltextfield()
            self.kcaltextfield.value = ""
            self.update()
        except ValueError:
            self.kcaltextfield.error_text = "\\(*O*)/ nicht verschreiben!!"
            self.update()

    def create_appbar(self):
        return ft.AppBar(
            title=ft.Text("Nahrungstracker"),
            actions=[
                ft.PopupMenuButton(
                    items=[ft.PopupMenuItem(text="Reset", on_click=self.resethistory)]
                )
            ],
        )

    def update_history(self):
        self.historylist.clear()
        for h in self.history:
            self.historylist.insert(0,
                ft.Container(content=
                    ft.Column(controls=[
                        ft.Row(
                            [ft.Text(str(f"{h[2][3]}:{h[2][4]}:{h[2][5]} {h[2][2]}.{h[2][1]}.{h[2][0]}"))],
                            alignment=ft.MainAxisAlignment.END
                        ),
                        ft.Row(
                            [ft.Text(str(f"({h[1]} kcal) {h[0]}"), size=20)],
                            alignment=ft.MainAxisAlignment.START,
                        )]),
                    bgcolor=ft.colors.INDIGO_600,  # "#226666",
                    border_radius=5,
                    padding=6
                )
            )

    def resethistory(self, e):
        print("pressed")
        def reset(e):
            self.kcaltext.value = "0"
            self.kjtext.value = "0"
            self.history = []
            file_handler.save(self.settings, self.history)
            self.update_history()
            self.history_container.content = ft.Column(self.historylist)
            self.dialog.open = False
            # beide updaten
            self.page.update()  
            self.update()  

        def close(e):
            self.dialog.open = False
            # beide updaten
            self.page.update()  
            self.update()

        self.dialog = ft.AlertDialog(
            modal=True,
            title=ft.Text("Wirklich?"),
            content=ft.Text("Alle Daten werden aus deinem Verlauf gelöscht!!!"),
            actions=[
                ft.TextButton("ja", on_click=reset),
                ft.TextButton("nein", on_click=close),
            ],
            actions_alignment=ft.MainAxisAlignment.END,
            on_dismiss=lambda e: print("Modal dialog dismissed!"),
        )
        self.page.dialog = self.dialog

        self.dialog.open = True
        self.page.update()

    def load_todays_kcal(self):
        today_date = get_time()[:3]
        print(today_date)

        todays_kcal = 0
        for entry in self.history:
            entry_date = entry[2][:3]  # Year, Month, Day part of the entry's timestamp
            if entry_date == today_date:
                todays_kcal += int(entry[1])

        self.kcaltext.value = str(todays_kcal)

